@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2022
+# Copyright (C) 2015-2023
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -38,10 +38,9 @@ from typing import (
 
 from telegram import Update
 from telegram._utils.defaultvalue import DEFAULT_TRUE, DefaultValue
-from telegram._utils.types import DVInput
+from telegram._utils.types import DVType
 from telegram._utils.warnings import warn
 from telegram.ext._application import ApplicationHandlerStop
-from telegram.ext._callbackcontext import CallbackContext
 from telegram.ext._callbackqueryhandler import CallbackQueryHandler
 from telegram.ext._choseninlineresulthandler import ChosenInlineResultHandler
 from telegram.ext._extbot import ExtBot
@@ -55,7 +54,7 @@ from telegram.ext._utils.types import CCT, ConversationDict, ConversationKey
 
 if TYPE_CHECKING:
     from telegram.ext import Application, Job, JobQueue
-_CheckUpdateType = Tuple[object, ConversationKey, BaseHandler, object]
+_CheckUpdateType = Tuple[object, ConversationKey, BaseHandler[Update, CCT], object]
 
 _logger = logging.getLogger(__name__)
 
@@ -71,7 +70,7 @@ class _ConversationTimeoutContext(Generic[CCT]):
     conversation_key: ConversationKey
     update: Update
     application: "Application[Any, CCT, Any, Any, Any, JobQueue]"
-    callback_context: CallbackContext
+    callback_context: CCT
 
 
 @dataclass
@@ -126,8 +125,8 @@ class ConversationHandler(BaseHandler[Update, CCT]):
 
     Warning:
         :class:`ConversationHandler` heavily relies on incoming updates being processed one by one.
-        When using this handler, :attr:`telegram.ext.Application.concurrent_updates` should be
-        :obj:`False`.
+        When using this handler, :attr:`telegram.ext.ApplicationBuilder.concurrent_updates` should
+        be set to :obj:`False`.
 
     Note:
         :class:`ConversationHandler` will only accept updates that are (subclass-)instances of
@@ -293,6 +292,7 @@ class ConversationHandler(BaseHandler[Update, CCT]):
     WAITING: ClassVar[int] = -3
     """:obj:`int`: Used as a constant to handle state when a conversation is still waiting on the
     previous :attr:`block=False <block>` handler to finish."""
+
     # pylint: disable=super-init-not-called
     def __init__(
         self,
@@ -307,7 +307,7 @@ class ConversationHandler(BaseHandler[Update, CCT]):
         name: str = None,
         persistent: bool = False,
         map_to_parent: Dict[object, object] = None,
-        block: DVInput[bool] = DEFAULT_TRUE,
+        block: DVType[bool] = DEFAULT_TRUE,
     ):
         # these imports need to be here because of circular import error otherwise
         from telegram.ext import (  # pylint: disable=import-outside-toplevel
@@ -319,25 +319,27 @@ class ConversationHandler(BaseHandler[Update, CCT]):
 
         # self.block is what the Application checks and we want it to always run CH in a blocking
         # way so that CH can take care of any non-blocking logic internally
-        self.block = True
+        self.block: DVType[bool] = True
         # Store the actual setting in a protected variable instead
-        self._block = block
+        self._block: DVType[bool] = block
 
-        self._entry_points = entry_points
-        self._states = states
-        self._fallbacks = fallbacks
+        self._entry_points: List[BaseHandler[Update, CCT]] = entry_points
+        self._states: Dict[object, List[BaseHandler[Update, CCT]]] = states
+        self._fallbacks: List[BaseHandler[Update, CCT]] = fallbacks
 
-        self._allow_reentry = allow_reentry
-        self._per_user = per_user
-        self._per_chat = per_chat
-        self._per_message = per_message
-        self._conversation_timeout = conversation_timeout
-        self._name = name
-        self._map_to_parent = map_to_parent
+        self._allow_reentry: bool = allow_reentry
+        self._per_user: bool = per_user
+        self._per_chat: bool = per_chat
+        self._per_message: bool = per_message
+        self._conversation_timeout: Optional[
+            Union[float, datetime.timedelta]
+        ] = conversation_timeout
+        self._name: Optional[str] = name
+        self._map_to_parent: Optional[Dict[object, object]] = map_to_parent
 
         # if conversation_timeout is used, this dict is used to schedule a job which runs when the
         # conv has timed out.
-        self.timeout_jobs: Dict[ConversationKey, "Job"] = {}
+        self.timeout_jobs: Dict[ConversationKey, "Job[Any]"] = {}
         self._timeout_jobs_lock = asyncio.Lock()
         self._conversations: ConversationDict = {}
         self._child_conversations: Set["ConversationHandler"] = set()
@@ -356,7 +358,7 @@ class ConversationHandler(BaseHandler[Update, CCT]):
                 stacklevel=2,
             )
 
-        all_handlers: List[BaseHandler] = []
+        all_handlers: List[BaseHandler[Update, CCT]] = []
         all_handlers.extend(entry_points)
         all_handlers.extend(fallbacks)
 
@@ -439,7 +441,7 @@ class ConversationHandler(BaseHandler[Update, CCT]):
                 )
 
     @property
-    def entry_points(self) -> List[BaseHandler]:
+    def entry_points(self) -> List[BaseHandler[Update, CCT]]:
         """List[:class:`telegram.ext.BaseHandler`]: A list of :obj:`BaseHandler` objects that can
         trigger the start of the conversation.
         """
@@ -452,7 +454,7 @@ class ConversationHandler(BaseHandler[Update, CCT]):
         )
 
     @property
-    def states(self) -> Dict[object, List[BaseHandler]]:
+    def states(self) -> Dict[object, List[BaseHandler[Update, CCT]]]:
         """Dict[:obj:`object`, List[:class:`telegram.ext.BaseHandler`]]: A :obj:`dict` that
         defines the different states of conversation a user can be in and one or more
         associated :obj:`BaseHandler` objects that should be used in that state.
@@ -464,7 +466,7 @@ class ConversationHandler(BaseHandler[Update, CCT]):
         raise AttributeError("You can not assign a new value to states after initialization.")
 
     @property
-    def fallbacks(self) -> List[BaseHandler]:
+    def fallbacks(self) -> List[BaseHandler[Update, CCT]]:
         """List[:class:`telegram.ext.BaseHandler`]: A list of handlers that might be used if
         the user is in a conversation, but every handler for their current state returned
         :obj:`False` on :meth:`check_update`.
@@ -641,7 +643,7 @@ class ConversationHandler(BaseHandler[Update, CCT]):
         new_state: asyncio.Task,
         application: "Application[Any, CCT, Any, Any, Any, JobQueue]",
         update: Update,
-        context: CallbackContext,
+        context: CCT,
         conversation_key: ConversationKey,
     ) -> None:
         try:
@@ -666,7 +668,7 @@ class ConversationHandler(BaseHandler[Update, CCT]):
         new_state: object,
         application: "Application[Any, CCT, Any, Any, Any, JobQueue]",
         update: Update,
-        context: CallbackContext,
+        context: CCT,
         conversation_key: ConversationKey,
     ) -> None:
         """Schedules a job which executes :meth:`_trigger_timeout` upon conversation timeout."""
@@ -685,7 +687,7 @@ class ConversationHandler(BaseHandler[Update, CCT]):
             _logger.exception("Failed to schedule timeout.", exc_info=exc)
 
     # pylint: disable=too-many-return-statements
-    def check_update(self, update: object) -> Optional[_CheckUpdateType]:
+    def check_update(self, update: object) -> Optional[_CheckUpdateType[CCT]]:
         """
         Determines whether an update should be handled by this conversation handler, and if so in
         which state the conversation currently is.
@@ -779,9 +781,9 @@ class ConversationHandler(BaseHandler[Update, CCT]):
     async def handle_update(  # type: ignore[override]
         self,
         update: Update,
-        application: "Application",
-        check_result: _CheckUpdateType,
-        context: CallbackContext,
+        application: "Application[Any, CCT, Any, Any, Any, Any]",
+        check_result: _CheckUpdateType[CCT],
+        context: CCT,
     ) -> Optional[object]:
         """Send the update to the callback for the current state and BaseHandler
 
@@ -899,7 +901,7 @@ class ConversationHandler(BaseHandler[Update, CCT]):
                 )
             self._conversations[key] = new_state
 
-    async def _trigger_timeout(self, context: CallbackContext) -> None:
+    async def _trigger_timeout(self, context: CCT) -> None:
         """This is run whenever a conversation has timed out. Also makes sure that all handlers
         which are in the :attr:`TIMEOUT` state and whose :meth:`BaseHandler.check_update` returns
         :obj:`True` is handled.
