@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2022
+# Copyright (C) 2015-2023
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -19,14 +19,14 @@
 """This module contains the DictPersistence class."""
 import json
 from copy import deepcopy
-from typing import Dict, Optional, cast
+from typing import Any, Dict, Optional, cast
 
 from telegram._utils.types import JSONDict
 from telegram.ext import BasePersistence, PersistenceInput
 from telegram.ext._utils.types import CDCData, ConversationDict, ConversationKey
 
 
-class DictPersistence(BasePersistence):
+class DictPersistence(BasePersistence[Dict[Any, Any], Dict[Any, Any], Dict[Any, Any]]):
     """Using Python's :obj:`dict` and :mod:`json` for making your bot persistent.
 
     Attention:
@@ -49,9 +49,9 @@ class DictPersistence(BasePersistence):
         The parameters and attributes ``store_*_data`` were replaced by :attr:`store_data`.
 
     Args:
-        store_data (:class:`PersistenceInput`, optional): Specifies which kinds of data will be
-            saved by this persistence instance. By default, all available kinds of data will be
-            saved.
+        store_data (:class:`~telegram.ext.PersistenceInput`, optional): Specifies which kinds of
+            data will be saved by this persistence instance. By default, all available kinds of
+            data will be saved.
         user_data_json (:obj:`str`, optional): JSON string that will be used to reconstruct
             user_data on creating this persistence. Default is ``""``.
         chat_data_json (:obj:`str`, optional): JSON string that will be used to reconstruct
@@ -71,8 +71,8 @@ class DictPersistence(BasePersistence):
 
             .. versionadded:: 20.0
     Attributes:
-        store_data (:class:`PersistenceInput`): Specifies which kinds of data will be saved by this
-            persistence instance.
+        store_data (:class:`~telegram.ext.PersistenceInput`): Specifies which kinds of data will
+            be saved by this persistence instance.
     """
 
     __slots__ = (
@@ -104,11 +104,11 @@ class DictPersistence(BasePersistence):
         self._bot_data = None
         self._callback_data = None
         self._conversations = None
-        self._user_data_json = None
-        self._chat_data_json = None
-        self._bot_data_json = None
-        self._callback_data_json = None
-        self._conversations_json = None
+        self._user_data_json: Optional[str] = None
+        self._chat_data_json: Optional[str] = None
+        self._bot_data_json: Optional[str] = None
+        self._callback_data_json: Optional[str] = None
+        self._conversations_json: Optional[str] = None
         if user_data_json:
             try:
                 self._user_data = self._decode_user_chat_data_from_json(user_data_json)
@@ -168,7 +168,7 @@ class DictPersistence(BasePersistence):
                 ) from exc
 
     @property
-    def user_data(self) -> Optional[Dict[int, Dict]]:
+    def user_data(self) -> Optional[Dict[int, Dict[Any, Any]]]:
         """:obj:`dict`: The user_data as a dict."""
         return self._user_data
 
@@ -180,7 +180,7 @@ class DictPersistence(BasePersistence):
         return json.dumps(self.user_data)
 
     @property
-    def chat_data(self) -> Optional[Dict[int, Dict]]:
+    def chat_data(self) -> Optional[Dict[int, Dict[Any, Any]]]:
         """:obj:`dict`: The chat_data as a dict."""
         return self._chat_data
 
@@ -192,7 +192,7 @@ class DictPersistence(BasePersistence):
         return json.dumps(self.chat_data)
 
     @property
-    def bot_data(self) -> Optional[Dict]:
+    def bot_data(self) -> Optional[Dict[Any, Any]]:
         """:obj:`dict`: The bot_data as a dict."""
         return self._bot_data
 
@@ -309,7 +309,7 @@ class DictPersistence(BasePersistence):
         self._conversations[name][key] = new_state
         self._conversations_json = None
 
-    async def update_user_data(self, user_id: int, data: Dict) -> None:
+    async def update_user_data(self, user_id: int, data: Dict[Any, Any]) -> None:
         """Will update the user_data (if changed).
 
         Args:
@@ -323,7 +323,7 @@ class DictPersistence(BasePersistence):
         self._user_data[user_id] = data
         self._user_data_json = None
 
-    async def update_chat_data(self, chat_id: int, data: Dict) -> None:
+    async def update_chat_data(self, chat_id: int, data: Dict[Any, Any]) -> None:
         """Will update the chat_data (if changed).
 
         Args:
@@ -337,7 +337,7 @@ class DictPersistence(BasePersistence):
         self._chat_data[chat_id] = data
         self._chat_data_json = None
 
-    async def update_bot_data(self, data: Dict) -> None:
+    async def update_bot_data(self, data: Dict[Any, Any]) -> None:
         """Will update the bot_data (if changed).
 
         Args:
@@ -389,21 +389,21 @@ class DictPersistence(BasePersistence):
         self._user_data.pop(user_id, None)
         self._user_data_json = None
 
-    async def refresh_user_data(self, user_id: int, user_data: Dict) -> None:
+    async def refresh_user_data(self, user_id: int, user_data: Dict[Any, Any]) -> None:
         """Does nothing.
 
         .. versionadded:: 13.6
         .. seealso:: :meth:`telegram.ext.BasePersistence.refresh_user_data`
         """
 
-    async def refresh_chat_data(self, chat_id: int, chat_data: Dict) -> None:
+    async def refresh_chat_data(self, chat_id: int, chat_data: Dict[Any, Any]) -> None:
         """Does nothing.
 
         .. versionadded:: 13.6
         .. seealso:: :meth:`telegram.ext.BasePersistence.refresh_chat_data`
         """
 
-    async def refresh_bot_data(self, bot_data: Dict) -> None:
+    async def refresh_bot_data(self, bot_data: Dict[Any, Any]) -> None:
         """Does nothing.
 
         .. versionadded:: 13.6
